package li.cil.oc.client.renderer.tileentity

import li.cil.oc.client.Textures
import li.cil.oc.common.tileentity.ServerRack
import li.cil.oc.util.RenderState
import net.minecraft.client.renderer.Tessellator
import net.minecraft.client.renderer.tileentity.TileEntitySpecialRenderer
import net.minecraft.tileentity.TileEntity
import net.minecraft.util.ResourceLocation
import net.minecraftforge.common.util.ForgeDirection
import org.lwjgl.opengl.GL11

object ServerRackRenderer extends TileEntitySpecialRenderer {
  private final val v1 = 2 / 16f
  private final val fs = 3 / 16f

  override def renderTileEntityAt(tileEntity: TileEntity, x: Double, y: Double, z: Double, f: Float) = {
    RenderState.checkError(getClass.getName + ".renderTileEntityAt: entering (aka: wasntme)")

    val rack = tileEntity.asInstanceOf[ServerRack]
    GL11.glPushAttrib(GL11.GL_ALL_ATTRIB_BITS)

    RenderState.disableLighting()
    RenderState.makeItBlend()

    GL11.glPushMatrix()

    GL11.glTranslated(x + 0.5, y + 0.5, z + 0.5)

    rack.yaw match {
      case ForgeDirection.WEST => GL11.glRotatef(-90, 0, 1, 0)
      case ForgeDirection.NORTH => GL11.glRotatef(180, 0, 1, 0)
      case ForgeDirection.EAST => GL11.glRotatef(90, 0, 1, 0)
      case _ => // No yaw.
    }

    GL11.glTranslated(-0.5, 0.5, 0.505)
    GL11.glScalef(1, -1, 1)

    if (rack.anyRunning) {
      for (i <- 0 until rack.getSizeInventory if rack.isRunning(i)) {
        renderFrontOverlay(Textures.blockRackFrontOn, i)
      }
      for (i <- 0 until rack.getSizeInventory if System.currentTimeMillis() - rack.lastAccess(i) < 400 && rack.world.rand.nextDouble() > 0.1) {
        renderFrontOverlay(Textures.blockRackFrontActivity, i)
      }
    }
    if (rack.anyErrored) {
      for (i <- 0 until rack.getSizeInventory if rack.hasErrored(i) && RenderUtil.shouldShowErrorLight(rack.hashCode * (i + 1))) {
        renderFrontOverlay(Textures.blockRackFrontError, i)
      }
    }

    RenderState.enableLighting()

    GL11.glPopMatrix()
    GL11.glPopAttrib()

    RenderState.checkError(getClass.getName + ".renderTileEntityAt: leaving")
  }

  private def renderFrontOverlay(texture: ResourceLocation, i: Int): Unit = {
    bindTexture(texture)

    val l = v1 + i * fs
    val h = v1 + (i + 1) * fs

    val t = Tessellator.instance
    t.startDrawingQuads()
    t.addVertexWithUV(0, h, 0, 0, h)
    t.addVertexWithUV(1, h, 0, 1, h)
    t.addVertexWithUV(1, l, 0, 1, l)
    t.addVertexWithUV(0, l, 0, 0, l)
    t.draw()
  }
}